@@ -1,8 +1,8 @@
-#############################################################################################################
+################################################################################
 # Author :
-#   Florian Rohart, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
-#   Kim-Anh Le Cao, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
-#   Benoit Gautier, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
+#   Florian Rohart,
+#   Kim-Anh Le Cao,
+#   Benoit Gautier,
 #
 # created: 22-04-2015
 # last modified: 04-10-2017
@@ -22,45 +22,37 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#############################################################################################################
+################################################################################
 
 
-# ========================================================================================================
-# Internal helpers functions to run "mixOmics" and "internal_mint.block" functions
-# ========================================================================================================
+# =============================================================================
+# Internal helpers functions to run "mixOmics" and "internal_mint.block"
+# =============================================================================
 
-# Some of these functions have been borrowed from the RGCCA package, as indicated below
+# Some of these functions have been borrowed from the RGCCA package,
+#   as indicated below
 
 # --------------------------------------
-# study_split: used in 'internal_mint.block.R' and 'predict.mint.block.pls.R'
+# get.weights: used in (MCV.)(block.)(s)pls(da).R
 # --------------------------------------
-get.weights = function(variates, indY)
+#' Get block weights for block analyses
+#'
+#' weights are calculated as correlation of block components with those of
+#' Y. Values saved in objects and used by \code{internal.predict.DA}
+#'
+#' @param variates Named list of variates from internal_wrapper.mint.block
+#' @param indY Integer, the Y matrix index from internal_wrapper.mint.block
+#'
+#' @return named vector of average correlations b/w block components and Y components
+#' @noRd
+get.weights <- function(variates, indY)
 {
-    ncomp = min(sapply(variates, ncol))
-    x.xList <- list()
-    compt = 1
-    for(comp in 1:ncomp)
-    {
-        for(i in 1:length(variates)){
-            corDat <- rep(0, length(variates))
-            names(corDat) <- paste("cor", names(variates)[i], names(variates), sep = "_")
-            for(j in 1:length(variates)){
-                corDat[j] <- as.numeric(cor(variates[[i]][,comp], variates[[j]][,comp]))
-            }
-            x.xList[[compt]] <- corDat
-            compt = compt +1
-        }
-    }
-    corMat.diablo <- do.call(rbind, x.xList)
-    rownames(corMat.diablo) <- paste(names(variates),".comp",rep(1:ncomp,each=length(variates)),sep="")
-    colnames(corMat.diablo) <- names(variates)
-    
-    temp = matrix(corMat.diablo[,indY],ncol=ncomp)
-    correlation = apply(temp, 1, function(x){mean(abs(x))})[1:length(variates)]
-    names(correlation) = names(variates)
-    
-    correlation = correlation[-indY]
-    return(correlation)
+    ## for all X blocks, and for i=1:ncomp calculate correlation of block_component_i and Y_component_i
+    block_correlation_with_Y <- lapply(variates[-indY], function(x) {
+        diag(cor(x, variates[[indY]]))
+    })
+    block_correlation_with_Y <- data.frame(t(data.frame(block_correlation_with_Y)))
+    return(block_correlation_with_Y)
     
 }
 
@@ -68,60 +60,95 @@ get.weights = function(variates, indY)
 # --------------------------------------
 # study_split: used in 'internal_mint.block.R' and 'predict.mint.block.pls.R'
 # --------------------------------------
-study_split = function(data, study)
+#' divides a data matrix in a list of matrices defined by a factor
+#' 
+#' \code{study_split} divides a data matrix in a list of matrices defined by a
+#' \code{study} input.
+#' 
+#' 
+#' @usage study_split(data, study)
+#' @param data numeric matrix of predictors
+#' @param study grouping factor indicating which samples are from the same
+#' study
+#' @return \code{study_split} simply returns a list of the same length as the
+#' number of levels of \code{study} that contains sub-matrices of \code{data}.
+#' @author Florian Rohart, Al J Abadi
+#' @seealso \code{\link{mint.pls}}, \code{\link{mint.spls}},
+#' \code{\link{mint.plsda}}, \code{\link{mint.splsda}}.
+#' @keywords regression multivariate
+#' @export
+#' @examples
+#' data(stemcells)
+#' data = stemcells$gene
+#' exp = stemcells$study
+#' 
+#' data.list = study_split(data, exp)
+#' 
+#' names(data.list)
+#' lapply(data.list, dim)
+#' table(exp)
+study_split <- function(data, study)
 {
-    #data should be a matrix
-    if(!any(class(data) == "matrix"))
-    data = as.matrix(data)
-    
-    M = length(levels(study))
-    
-    #---------------------- split data
-    if(M>1)
-    {
-        data.list.study = split.data.frame(data,study)
-    } else {
-        data.list.study = list(data)
-        names(data.list.study) = levels(study)
-    }
-
-    result = data.list.study
-    return(invisible(result))
+    return(
+        invisible(
+            split.data.frame(as.matrix(data), study)
+        )
+    )
 }
 
 
 # --------------------------------------
 # soft_thresholding: used in sparsity (below)
 # --------------------------------------
-# x: vector
-# nx: number of entries to put to zero
-
-soft_thresholding_L1 = function(x,nx)
+#' Soft-thresholding of loading vectors
+#'
+#' @param x A vector of loadings
+#' @param nx Integer, number of features whose loadings are to be set to 0
+#'
+#' @return A numeric vector of the penalised loadings
+#'
+#' @details If the number of features to be dropped are greater than 0, the
+#'   procedure does so by setting the smallest weight by absolute value to 0.
+#'   The weights of the remaining features are then shrunk towards zero by the
+#'   absolute value of the greatest weight eliminated.
+#' @noRd
+#' @examples
+#' x1 = c(0.9, -1.2, 0.23, 0.11)
+#' x2 = c(0.9, -1.2, 0.23, 0.89)
+#' 
+#' soft_thresholding_L1(x = x1, nx = 2)
+#' #> 0.67 -0.97  0.00  0.00
+#' 
+#' soft_thresholding_L1(x = x2, nx = 2)
+#' #> 0.01 -0.31  0.00  0.00
+soft_thresholding_L1 <- function(x, nx)
 {
-    #selection on a (loadings.X). modified on 19/02/15 to make sure that a!=0
-    if (nx!=0)
+    if (nx > 0)
     {
-        absa = abs(x)
-        if (any(rank(absa, ties.method = "max") <= nx))
+        abs.a = abs(x)
+        select_feature <- rank(abs.a, ties.method = "max") > nx
+        if (!all(select_feature))
         {
-            x = ifelse(rank(absa, ties.method = "max") <= nx, 0,
-                        sign(x) * (absa - max(absa[rank(absa, ties.method = "max") <= nx])))
+            x <- ifelse(test = select_feature, 
+                        yes = sign(x) * (abs.a - max(abs.a[!select_feature])), 
+                        no = 0)
         }
     }
     
-    x
+    return(x)
 }
 
-# ----------------------------------------------------------------------------------------------------------
-# soft.threshold() - soft-thresholds a vector such that the L1-norm constraint is satisfied.
-# ----------------------------------------------------------------------------------------------------------
-soft.threshold = function (x, sumabs = 1)
-return(soft(x, BinarySearch(x, sumabs)))
+# -----------------------------------------------------------------------------
+# soft.threshold() - soft-thresholds a vector such that
+# the L1-norm constraint is satisfied.
+# -----------------------------------------------------------------------------
+soft.threshold <- function (x, sumabs = 1)
+    return(soft(x, BinarySearch(x, sumabs)))
 
-BinarySearch = function(argu,sumabs)
+BinarySearch <- function(argu,sumabs)
 {
     if (norm2(argu)==0 || sum(abs(argu/norm2(argu)))<=sumabs)
-    return(0)
+        return(0)
     
     lam1 = 0
     lam2 = max(abs(argu))-1e-5
@@ -136,7 +163,7 @@ BinarySearch = function(argu,sumabs)
             lam1 = (lam1+lam2)/2
         }
         if ((lam2-lam1)<1e-10)
-        return((lam1+lam2)/2)
+            return((lam1+lam2)/2)
         
         iter = iter+1
     }
@@ -144,13 +171,13 @@ BinarySearch = function(argu,sumabs)
     return((lam1+lam2)/2)
 }
 
-soft = function(x,d) return(sign(x)*pmax(0, abs(x)-d))
+soft <- function(x,d) return(sign(x)*pmax(0, abs(x)-d))
 
-norm2 = function(vec)
+norm2 <- function(vec)
 {
     a = sqrt(sum(vec^2))
     if (a == 0)
-    a = .05
+        a = .05
     
     return(a)
 }
@@ -159,7 +186,7 @@ norm2 = function(vec)
 # --------------------------------------
 # sparsity function: used in 'internal_mint.block.R'
 # --------------------------------------
-sparsity=function(loadings.A, keepA, penalty=NULL)
+sparsity <- function(loadings.A, keepA, penalty=NULL)
 {
     
     if (!is.null(keepA)) {
@@ -168,7 +195,7 @@ sparsity=function(loadings.A, keepA, penalty=NULL)
     } else if (!is.null(penalty)) {
         loadings.A = soft.threshold(loadings.A, penalty)
     }
-
+    
     return(loadings.A)
 }
 
@@ -177,13 +204,15 @@ sparsity=function(loadings.A, keepA, penalty=NULL)
 # --------------------------------------
 # scaling with or without bias: used in mean_centering_per_study (below)
 # --------------------------------------
-scale.function_old=function(temp, scale = TRUE) # problem: divide by n instead of n-#NA
+scale.function_old <- function(temp, scale = TRUE)
+    # problem: divide by n instead of n-#NA
 {
     meanX = colMeans(temp, na.rm = TRUE)
     data.list.study.scale_i = t(t(temp) - meanX)
     if (scale)
     {
-        sqrt.sdX = sqrt(colSums(data.list.study.scale_i^2, na.rm = TRUE) / (nrow(temp) - 1))
+        sqrt.sdX = sqrt(colSums(data.list.study.scale_i^2, na.rm = TRUE) /
+                            (nrow(temp) - 1))
         data.list.study.scale_i = t(t(data.list.study.scale_i) / sqrt.sdX)
     } else {
         sqrt.sdX = NULL
@@ -193,36 +222,49 @@ scale.function_old=function(temp, scale = TRUE) # problem: divide by n instead o
     #if (sum(is.na.data) > 0)
     #data.list.study.scale_i[is.na.data] = 0
     
-    out = list(data_scale=data.list.study.scale_i, meanX=meanX, sqrt.sdX=sqrt.sdX)
+    out = list(data_scale=data.list.study.scale_i, meanX=meanX,
+               sqrt.sdX=sqrt.sdX)
     return(out)
 }
 
 # --------------------------------------
-# scaling, using colSds from library(matrixStats), used in mean_centering_per_study (below)
+# scaling, using colSds from library(matrixStats),
+# used in mean_centering_per_study (below)
 # --------------------------------------
-scale.function=function(temp, scale = TRUE)
+#' @importFrom matrixStats colSds 
+scale.function <- function(temp, scale = TRUE)
 {
     meanX = colMeans(temp, na.rm = TRUE)
     
     if (scale)
     {
         sqrt.sdX = colSds(temp,  na.rm=TRUE)
-        data.list.study.scale_i = t( (t(temp)-meanX) / sqrt.sdX)
+        # first possiblity: scale(), too long
+        # second possibility: matrix approach: transpose is too long
+        # data.list.study.scale_i = t( (t(temp)-meanX) / sqrt.sdX)
+        
+        # third possibility (ell.equal=>TRUE)
+        data.list.study.scale_i = temp-rep(1, nrow(temp)) %*% t(meanX)
+        data.list.study.scale_i = data.list.study.scale_i /
+            rep(1, nrow(temp)) %*% t(sqrt.sdX)
+        
         
         ind = which(sqrt.sdX == 0) # scaling can creates NA
         if(length(ind) >0)
-        data.list.study.scale_i[,ind] = 0
+            data.list.study.scale_i[,ind] = 0
         
     } else {
         sqrt.sdX = NULL
-        data.list.study.scale_i = t( (t(temp)-meanX))
+        # data.list.study.scale_i = t( (t(temp)-meanX)) # too long bc t()
+        data.list.study.scale_i = temp-rep(1, nrow(temp)) %*% t(meanX)
     }
     
     #is.na.data = is.na(data.list.study.scale_i)
     #if (sum(is.na.data) > 0)
     #data.list.study.scale_i[is.na.data] = 0
     
-    out = list(data_scale=data.list.study.scale_i, meanX=meanX, sqrt.sdX=sqrt.sdX)
+    out = list(data_scale=data.list.study.scale_i, meanX=meanX,
+               sqrt.sdX=sqrt.sdX)
     return(out)
 }
 
@@ -230,20 +272,20 @@ scale.function=function(temp, scale = TRUE)
 # --------------------------------------
 # Mean centering/scaling per study: used in 'internal_mint.block.R'
 # --------------------------------------
-mean_centering_per_study=function(data, study, scale)
+mean_centering_per_study <- function(data, study, scale)
 {
     
     M = length(levels(study))   # number of groups
     # split the data
     data.list.study = study_split(data, study)
-
+    
     # center and scale data per group, and concatene the data
     res = lapply(data.list.study, scale.function, scale = scale)
     
     meanX = lapply(res, function(x){x[[2]]})
     sqrt.sdX = lapply(res, function(x){x[[3]]})
     rownames.study = lapply(res, function(x){rownames(x[[1]])})
-
+    
     #rename rows and cols of concatenated centered (and/or scaled) data
     #colnames(concat.data) = colnames(data) #already done
     
@@ -258,24 +300,25 @@ mean_centering_per_study=function(data, study, scale)
     }
     if (M > 1)
     {
-        for (m in 1:M)
+        for (m in seq_len(M))
         {
             attr(concat.data,paste0("means:", levels(study)[m])) = meanX[[m]]
             if(scale)
             {
-                attr(concat.data,paste0("sigma:", levels(study)[m])) = sqrt.sdX[[m]]
+                attr(concat.data,paste0("sigma:", levels(study)[m])) =
+                    sqrt.sdX[[m]]
             } else {
                 attr(concat.data,paste0("sigma:", levels(study)[m])) = NULL
             }
         }
+    }
+
+    attr(concat.data,"scaled:center") = meanX[[1]]
+    if (scale)
+    {
+        attr(concat.data,"scaled:scale") = sqrt.sdX[[1]]
     } else {
-        attr(concat.data,"scaled:center") = meanX[[1]]
-        if (scale)
-        {
-            attr(concat.data,"scaled:scale") = sqrt.sdX[[1]]
-        } else {
-            attr(concat.data,"scaled:scale") = NULL
-        }
+        attr(concat.data,"scaled:scale") = NULL
     }
     
     return(list(concat.data=concat.data, rownames.study=rownames.study))
@@ -285,10 +328,10 @@ mean_centering_per_study=function(data, study, scale)
 # --------------------------------------
 # l2.norm: used in 'internal_mint.block.R'
 # --------------------------------------
-l2.norm=function(x)
+l2.norm <- function(x)
 {
     if (!is.vector(x))
-    stop("x has to be a vector")
+        stop("x has to be a vector")
     
     out = x / drop(sqrt(crossprod(x)))
 }
@@ -297,10 +340,10 @@ l2.norm=function(x)
 # tau.estimate() - Estimation of tau accoring to Strimmer formula
 # ---------------------------------------------------
 #used in 'internal_mint.block.R'
-tau.estimate = function (x)
+tau.estimate <- function (x)
 {
     if (is.matrix(x) == TRUE && is.numeric(x) == FALSE)
-    stop("The data matrix must be numeric!")
+        stop("The data matrix must be numeric!")
     
     p = NCOL(x)
     n = NROW(x)
@@ -319,14 +362,14 @@ tau.estimate = function (x)
 }
 
 
-#############################################################################################################
+################################################################################
 # Functions acquired from RGCCA R-library
-#############################################################################################################
-# ----------------------------------------------------------------------------------------------------------
+################################################################################
+# ------------------------------------------------------------------------------
 # cov2() - Compute biased and unbiased covariance and variance estimates
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # used in 'internal_mint.block.R'
-cov2 = function (x, y = NULL, bias = FALSE) {
+cov2 <- function (x, y = NULL, bias = FALSE) {
     n = NROW(x)
     if (is.null(y)) {
         x = as.matrix(x)
@@ -346,22 +389,22 @@ cov2 = function (x, y = NULL, bias = FALSE) {
 }
 
 
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # miscrossprod() - Compute cross-product between vectors x and y
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # used in 'internal_mint.block.R'
-miscrossprod = function (x, y) {
+miscrossprod <- function (x, y) {
     d.p = sum(drop(x) * drop(y), na.rm = TRUE)
     #d.p = as.vector(d.p)/norm2(d.p)     ## change made
     return(d.p)
 }
 
 
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # deflation()
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # used in defl.select (below)
-deflation = function(X, y, misdata.q, is.na.A.q, ind.NA){
+deflation <- function(X, y, misdata.q, is.na.A.q, ind.NA){
     # Computation of the residual matrix R
     # Computation of the vector p.
     
@@ -389,7 +432,8 @@ deflation = function(X, y, misdata.q, is.na.A.q, ind.NA){
         
         if(length(ind.NA)>0)
         {
-            temp = drop(y) %o% rep(1, length(ind.NA)) # should be n*p, but we limit it to n* where there's NA
+            temp = drop(y) %o% rep(1, length(ind.NA))
+            # should be n*p, but we limit it to n* where there's NA
             temp[is.na.A.q[,ind.NA,drop=FALSE]] = 0
             d.loadings.A.norm[ind.NA] = apply(temp,2, crossprod)
         }
@@ -398,7 +442,7 @@ deflation = function(X, y, misdata.q, is.na.A.q, ind.NA){
         # we can have 0/0, so we put 0
         a = is.na(p)
         if (any(a))
-        p[a] = 0
+            p[a] = 0
         
     } else {
         p <- crossprod(X,y) / as.vector(crossprod(y))
@@ -409,36 +453,46 @@ deflation = function(X, y, misdata.q, is.na.A.q, ind.NA){
 }
 
 
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # defl.select() - computes residual matrices
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # used in 'internal_mint.block.R'
-defl.select = function(yy, rr, nncomp, nn, nbloc, indY = NULL, mode = "canonical", aa = NULL, misdata, is.na.A, ind.NA) { ### Start: Add new parameter for estimation classic mode
+defl.select <- function(yy, rr, nncomp, nn, nbloc, indY = NULL,
+                        mode = "canonical", aa = NULL, misdata, is.na.A, ind.NA) {
+    ### Start: Add new parameter for estimation classic mode
     #save(list=ls(),file="temp2.Rdata")
-    resdefl = NULL
-    pdefl = NULL
-    for (q in 1 : nbloc) {
-        # for each block we create missing data parameters to be passed to the deflation()
+    resdefl = pdefl = vector("list",length=nbloc)
+    for (q in seq_len(nbloc)) {
+        # for each block we create missing data parameters to be passed to
+        #  deflation()
         if(misdata[q])
         {
             is.na.A.q = is.na.A[[q]]
         } else {
             is.na.A.q = NULL
         }
-        ### Start: insertion of new deflations (See La regression PLS Theorie et pratique p204 (Chap 11))
+        ### Start: insertion of new deflations
+        # (See La regression PLS Theorie et pratique p204 (Chap 11))
         if ( nn <= nncomp[q] ) {
-            if ((mode == "canonical") || (q != indY)) { #deflation of each block independently from the others, except indY
-                defltmp = deflation(rr[[q]], yy[ , q], misdata[q], is.na.A.q, ind.NA[[q]])
+            if ((mode == "canonical") || (q != indY)) {
+                #deflation of each block independently from the others,
+                # except indY
+                defltmp = deflation(rr[[q]], yy[ , q], misdata[q],
+                                    is.na.A.q, ind.NA[[q]])
+                #save(list=ls(),file="temp.Rdata")
                 resdefl[[q]] = defltmp$R
                 pdefl[[q]]   = defltmp$p
             } else if (mode == "classic") {
-                resdefl[[q]] = Reduce("+", lapply(c(1:nbloc)[-q], function(x) {rr[[q]] - yy[ ,x]%*%t(aa[[q]])}))/(nbloc-1)
+                resdefl[[q]] = Reduce("+", lapply(c(seq_len(nbloc))[-q], function(x)
+                {rr[[q]] - yy[ ,x]%*%t(aa[[q]])}))/(nbloc-1)
                 pdefl[[q]]   =  rep(0,NCOL(rr[[q]]))
             } else if (mode == "invariant") { #no deflation
                 resdefl[[q]] = rr[[q]]
                 pdefl[[q]]   =  rep(0,NCOL(rr[[q]]))
             } else if (mode == "regression") {
-                resdefl[[q]] = Reduce("+", lapply(c(1:nbloc)[-q], function(x) {deflation(rr[[q]],yy[, x], misdata[q], is.na.A.q, ind.NA[[q]])$R}))/(nbloc-1)
+                resdefl[[q]] = Reduce("+", lapply(c(seq_len(nbloc))[-q], function(x)
+                {deflation(rr[[q]],yy[, x], misdata[q], is.na.A.q,
+                           ind.NA[[q]])$R}))/(nbloc-1)
                 pdefl[[q]]   =  rep(0,NCOL(rr[[q]]))
             }
             ### End: insertion of new deflations
@@ -452,28 +506,34 @@ defl.select = function(yy, rr, nncomp, nn, nbloc, indY = NULL, mode = "canonical
     return(list(resdefl=resdefl,pdefl=pdefl))
 }
 
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # initsvd() - performs SVD on matrix X
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # used in 'internal_mint.block.R'
-initsvd = function (X) {
+
+#' @importFrom rARPACK svds
+initsvd <- function (X) {
     n = NROW(X)
     p = NCOL(X)
     
     if(p>3) #use svds
     {
-        ifelse(n >= p, return(svds(X, k=1, nu = 1, nv = 1)$v), return(svds(X, k=1, nu = 1, nv = 1)$u))
+        ifelse(n >= p, return(svds(X, k=1, nu = 1, nv = 1)$v),
+               return(svds(X, k=1, nu = 1, nv = 1)$u))
         
     } else {
-        ifelse(n >= p, return(svd(X, nu = 0, nv = 1)$v), return(svd(X, nu = 1, nv = 0)$u))
+        ifelse(n >= p, return(svd(X, nu = 0, nv = 1)$v),
+               return(svd(X, nu = 1, nv = 0)$u))
         
     }
 }
 
-# ----------------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 # init svd
-# ----------------------------------------------------------------------------------------------------------
-initialisation_by_svd = function(A, indY = NULL, misdata, is.na.A = NULL, init = "svd")
+# ------------------------------------------------------------------------------
+#' @importFrom rARPACK svds
+initialisation_by_svd <- function(A, indY = NULL, misdata, is.na.A = NULL,
+                                  init = "svd")
 {
     
     J = length(A)
@@ -483,24 +543,27 @@ initialisation_by_svd = function(A, indY = NULL, misdata, is.na.A = NULL, init =
     {
         
         # same step with or without NA, as they are already replaced by 0
-        M = lapply(c(1:J)[-indY], function(x){crossprod(A[[x]], A[[indY]])})
+        M = lapply(c(seq_len(J))[-indY], function(x){crossprod(A[[x]], A[[indY]])})
         #ssvd faster with svds, only if more than 3 column, otherwise break down
-        svd.M = lapply(M, function(x){if(ncol(x)>3) {svds(x, k=1, nu = 1, nv = 1)} else {svd(x, nu = 1, nv = 1)}})
+        svd.M = lapply(M, function(x){if(ncol(x)>3)
+        {svds(x, k=1, nu = 1, nv = 1)} else {svd(x, nu = 1, nv = 1)}})
         
-        loadings.A[c(1:J)[-indY]] = lapply(1:length(M), function(x){svd.M[[x]]$u})
+        loadings.A[c(seq_len(J))[-indY]] = lapply(seq_len(length(M)), function(x)
+        {svd.M[[x]]$u})
         loadings.A[[indY]] = svd.M[[1]]$v
         
     } else if (init=="svd.single") {
         
-        alpha =  lapply(1 : J, function(y){initsvd(A[[y]])})
-
-        for (j in 1:J)
+        alpha =  lapply(seq_len(J), function(y){initsvd(A[[y]])})
+        
+        for (j in seq_len(J))
         {
             if (nrow(A[[j]]) >= ncol(A[[j]]))
             {
                 loadings.A[[j]] = alpha[[j]]
             } else {
-                alpha[[j]] = drop(1/sqrt( t(alpha[[j]]) %*% A[[j]] %*% (t(A[[j]]) %*% alpha[[j]]))) * alpha[[j]]
+                alpha[[j]] = drop(1/sqrt( t(alpha[[j]]) %*% A[[j]] %*%
+                                              (t(A[[j]]) %*% alpha[[j]]))) * alpha[[j]]
                 
                 loadings.A[[j]] = crossprod(A[[j]],alpha[[j]])
             }
@@ -508,8 +571,8 @@ initialisation_by_svd = function(A, indY = NULL, misdata, is.na.A = NULL, init =
     } else {
         stop("init should be either 'svd' or 'svd.single'.")
     }
-
+    
     return(loadings.A)
-
+    
 }
 
